package asia.redact.bracket.properties;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Implement a SAXParser handler to consume the SAX events and turn that into a Properties file.
 *  It would be cleaner to use Dom4J or something like that, but I wanted to
 * avoid additional dependencies where I could.
 * 
 * @author Dave
 *
 */

class BracketPropertiesSAXHandler extends DefaultHandler {

	Properties props;
	int count;
	
	StringBuffer keyBuf;
	Stack<String> keyStack;
	List<String> comments;
	String separator;
	List<String> values;
	
	boolean commentOn;
	boolean separatorOn;
	boolean valueOn;
	
	public BracketPropertiesSAXHandler() {
		this.props = Properties.Factory.getInstance();
		keyStack = new Stack<String>();
		comments = new ArrayList<String>();
		values = new ArrayList<String>();
	}
	
	public Properties getResult() {
		return props;
	}
	
	public void characters(char[] buffer, int start, int length) {
        String temp = new String(buffer, start, length);
        if(commentOn){
        	comments.add(temp);
        }else if(separatorOn){
        	separator = temp;
        }else if(valueOn){
        	values.add(temp);
        }
	}

	
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		
		// test the initial (wrapping) element, which should be "properties"
		if(count == 0) {
			if(qName.equals("properties")){
				String namespace = attributes.getValue("xmlns");
				if(namespace == null || !namespace.equals("http://code.google.com/p/bracket-properties")){
					throw new RuntimeException("Bad namespace, attribute xmlns must be set to 'http://code.google.com/p/bracket-properties'");
				}
			}else{
				throw new RuntimeException("Bracket properties serialized with xml must have a top level element called 'properties'");
			}
		}else{
			
			if(qName.equals("comment")){
				commentOn = true; 
				return;
			}
			
			if(qName.equals("separator")){
				separatorOn = true; 
				return;
			}
			
			if(qName.equals("value")){
				valueOn = true; 
				return;
			}
			
			
			// ok, must be part of a key - do the work of collecting elements for a key
			
			keyStack.push(qName);
		}
     		
		count++;
	}

	public void endElement(String uri, String localName, String qName) throws SAXException {
		 
		if(qName.equals("comment")){
			commentOn = false; 
			return;
		}
		
		if(qName.equals("separator")){
			separatorOn = false; 
			return;
		}
		
		if(qName.equals("value")){
			valueOn = false; 
			return;
		}
		
		// ending qName must be part of key; check if values are defined
		
		if(values.size() > 0 && separator != null){
			// we have values and a separator - create a key from the stack, popping one item
			
			 keyBuf = new StringBuffer();
			 for(String part: keyStack){
				 if(keyBuf.length() > 0) keyBuf.append(".");
				 keyBuf.append(part);
			 }
			// keyBuf.append(".");
			// keyBuf.append(qName);
			 ValueModel model = new ValueModel();
			 model.setSeparator(separator.charAt(0));
			 for(String value: values){
				model.addValue(value);
			 }
			 for(String comment: comments){
				model.addComment(comment);
			 }
				
			 props.getPropertyMap().put(keyBuf.toString(), model);
				
			 // now clean up
				
			 keyBuf = null;
			 values.clear();
			 comments.clear();
			 separator = null;
			 
			 // pop the topmost item
			 keyStack.remove(qName);
			 
		}else{
			keyStack.remove(qName);
		}
	}
}
